#ifndef NETWORK_H
#define NETWORK_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <unistd.h>
#include <fstream>

class Network
{
public:
  Network(string host, unsigned long port)
  {
    struct hostent* he = gethostbyname(host.c_str());
    mSocket = socket(PF_INET, SOCK_STREAM, getprotobyname("tcp")->p_proto);
    struct sockaddr_in sin = {0};
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);
    sin.sin_addr.s_addr = *(long*)(he->h_addr_list[0]);
    
    if (connect(mSocket, (sockaddr*)&sin, sizeof(sin)))
    { 
      cout << "Unable to connect to " << host << "(" << errno << ")" << endl;
      exit(2);
    }

    ofs.open("net.log");
  }

  ~Network()
  {
    close(mSocket);
  }

  Network& operator<<(const string& s)
  {
    string ts = s + "\n";
    if (-1 == send(mSocket, ts.c_str(), ts.size(), MSG_DONTWAIT))
      exit(3);
    ofs << ts << flush;
    return *this;
  }

  Network& operator>>(string& s)
  {
    char c = 0;
    s = "";
    while (c != '\n')
    {
      *this >> c;
      if (c != '\n')
        s += c;
    }

    return *this;
  }

  Network& operator>>(char& c)
  {
    if (-1 == recv(mSocket, &c, sizeof(char), 0))
      exit(4);
    ofs << c << flush;
    return  *this;
  }

  Network& operator>>(int& i)
  {
    i = 0;
    char c = '0';
    while (isdigit(c))
    {
      *this >> c;
      if (isdigit(c))
      {
        i *= 10;
        i += c - '0';
      }
    }

    return *this;
  }
  
private:
  string mHost;
  int    mPort;
  int    mSocket;
  ofstream ofs;
};

#endif

