#include "Player.h"

void Player::run()
{
  mBoard.addRobot(this);
  readServerResponse();

  while (1)
  {
    readPackages();
    makeMove();
    readServerResponse();
  }
}

void Player::readServerResponse()
{
  string s;
  mNet >> s;
  istringstream iss(s);
  char c = 0;
  int id;
  bool foundme = false;
  while (iss.good())
  {
    int num;
    int x, y;
    c = 0;
    iss >> c; // Reads the space and next char
    switch(c)
    {
      // Set position
      case 'X': iss >> x >> c >> y; 
                //cout << "Read:" << id << " at:" << x << "," << y << endl;
                mBoard.move(id, Point(x, y)); 
                break;

      // Move cardinal directions
      case 'N': mBoard.moveNorth(id); 
                //cout << id << ":" << 'N' << endl; 
                break;
      case 'S': mBoard.moveSouth(id); 
                //cout << id << ":" << 'S' << endl; 
                break;
      case 'E': mBoard.moveEast(id); 
                //cout << id << ":" << 'E' << endl; 
                break;
      case 'W': mBoard.moveWest(id); 
                //cout << id << ":" << 'W' << endl; 
                break;

      // Handle pickup
      case 'P': iss >> num; mBoard.pickup(id, num); 
                //cout << id << " picked up " << num << endl; 
                break;
  
      // Handle drop
      case 'D': iss >> num; mBoard.drop(id, num); 
                //cout << id << " dropped " << num << endl; 
                break;

      // Next ID
      case '#': iss >> id; 
                break;
    }

    if (id == this->id())
      foundme = true;
  }

  if (!foundme)
  {
    cout << "I'm dead?" << endl;
    exit(20);
  }
}

void Player::readPackages()
{
  string s;
  mNet >> s;
  istringstream iss(s);
  mBoard.clearPackagesAt(pos());
  while (iss.good())
  {
    int id = -1;
    int x = -1;
    int y = -1;
    int w = -1;
    iss >> id >> x >> y >> w;

    if (x == 0 || y == 0)
    {
      cout << "Invalid response" << endl;
      exit(9);
    }

    if (id != -1 && x != -1 && y != -1 && w != -1)
      mBoard.addPackage(pos(), Package(id, Point(x, y), w));
  }
}

void Player::makeMove()
{
  if (mMoney <= 0)
  {
    cout << "No money" << endl;
    exit(8);
  }

  if (!mRoute.size())
  {
    if (mPkgs.size())
    {
      Path path = mBoard.getShortestPath(pos(), mPkgs[0].dest());
      for (int i = 1; i < mPkgs.size(); i++)
      {
        Path tst(mBoard.getShortestPath(pos(), mPkgs[i].dest()));
        if (tst.size() && tst.size() < path.size() ||
            tst.size() && !path.size())
          path = tst;
      }
      mRoute = path;
    }
    else
      mRoute = mBoard.getPathToClosestPackage(pos());
  }

retryroute:
  if (mRoute.size())
  {
    if (pos() != mRoute.currPos())
    {
      cout << "PUSHED?: Oops, not where I'm supposed to be. at" << pos() 
           << " should be at " << mRoute.currPos() << endl;
      mRoute = mBoard.getShortestPath(pos(), mRoute.dest());
      cout << "New route:" << endl;
      mRoute.print();
    }

    if (pos() != mRoute.dest())
    {
      ostringstream oss;
      int bid = 1;
      mRoute.inc();
      cout << "Moving from:" << pos() << " to " << mRoute.currPos() << endl;
      oss << bid << " Move " << mBoard.getDir(pos(), mRoute.currPos());
      mNet << oss.str();
  
      mTried = eMove;
      mMoney -= abs(bid);

      return;
    }
    else if (pos() == mRoute.dest())
    {
      // Make sure I'm really at my dest
      if (mRoute.dest() != pos())
      {
        mRoute = mBoard.getShortestPath(pos(), mRoute.dest());
        goto retryroute;
      }
  
      if (mPkgs.size())
      {
        cout << "Have pkgs" << endl;
        vector<int> dpkgs;
        for (int i = 0; i < mPkgs.size(); i++)
        {
          if (mPkgs[i].dest() == pos())
          {
            dpkgs.push_back(mPkgs[i].id());
          }
        }
  
        if (dpkgs.size())
        {
          ostringstream oss;
          int bid = 1;
          oss << bid << " Drop";
          for (int i = 0; i < dpkgs.size(); i++)
            oss << " " << dpkgs[i];
          mNet << oss.str();
          mMoney -= abs(bid);
          mTried = eDrop;
        }

        mRoute.clear();
  
        return;
      }
      else
      {
        cout << "Don't have pkgs" << endl;
        // Pickup Package
        vector<Package> pkgs(mBoard.getPackagesAt(pos()));
        if (!pkgs.size())
          mBoard.removeHome(pos());

        int availw = getAvailWeight();
        cout << "AvailW:" << availw << endl;
        map<int, vector<int> > ppkgs;
        // Get all packages
        for (int i = 0; i < pkgs.size(); i++)
        {
          cout << "Checking:" << pkgs[i].id() << ":" << pkgs[i].dest() << ":" << pkgs[i].weight() << endl;
          if (!mBoard.getShortestPath(pos(), pkgs[i].dest()).size())
          {
            continue;
          }

          ppkgs[pkgs[i].weight()].push_back(pkgs[i].id());
        }

        vector<int> appkgs;
        map<int, vector<int> >::reverse_iterator pit = ppkgs.rbegin();
        for (; pit != ppkgs.rend(); pit++)
        {
          while (pit->second.size() &&
                 availw - pit->first >= 0)
          {
            appkgs.push_back(pit->second.back());
            pit->second.pop_back();
            availw -= pit->first;
          }
        }

        if (appkgs.size())
        {
          ostringstream oss;
          int bid = 1;
          oss << bid << " Pick";
          for (int i = 0; i < appkgs.size(); i++)
            oss << " " << appkgs[i];
          mNet << oss.str();
          mTried = ePick;
          mMoney -= abs(bid);
          mRoute.clear();
          return;
        }
        else
        {
          mRoute = mBoard.getPathToClosestPackage(pos());
          goto retryroute;
        }
      }
    }
  }
  
  cout << "Nothing to do" << endl;
  mNet << "1 Drop";
  exit(10);
}

